package ru.crystals.test.plugins;

import org.slf4j.Logger;
import ru.crystals.pos.api.ext.loyal.dto.BonusAccrualResult;
import ru.crystals.pos.api.ext.loyal.dto.DiscountCancellationResult;
import ru.crystals.pos.api.ext.loyal.dto.LoyaltyResult;
import ru.crystals.pos.api.ext.loyal.dto.PostDiscountResult;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.FiscalizationFeedback;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.LoyProviderFeedback;
import ru.crystals.pos.api.ext.loyal.dto.auxiliaries.PreFiscalizationFeedback;
import ru.crystals.pos.api.plugin.LoyaltyPlugin;
import ru.crystals.pos.spi.IntegrationProperties;
import ru.crystals.pos.spi.annotation.Inject;
import ru.crystals.pos.spi.annotation.POSPlugin;
import ru.crystals.pos.spi.equipment.CustomerDisplay;
import ru.crystals.pos.spi.receipt.Receipt;
import ru.crystals.test.plugins.builders.SlipBuilder;
import ru.crystals.test.plugins.data.AttributeEntry;
import ru.crystals.test.plugins.data.Discount;
import ru.crystals.test.plugins.data.ObjectFactory;
import ru.crystals.test.plugins.data.PluginResult;
import ru.crystals.test.plugins.data.PrintSlip;

import javax.annotation.PostConstruct;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import java.io.File;
import java.util.Collection;

@POSPlugin(id="ru.crystals.setapi.test.loyalty.plugin")
public class TestLoyaltyPlugin implements LoyaltyPlugin {

    @Inject
    private Logger logger;

    @Inject
    private IntegrationProperties integrationProperties;

    @Inject
    private CustomerDisplay display;

    private Unmarshaller unmarshaller;
    private ResponseReaderRoutine responseReader;
    private CustomerMessageRoutine displayRoutine = new CustomerMessageRoutine();

    @PostConstruct
    public void init() {
        JAXBContext context;
        try {
            context = JAXBContext.newInstance(ObjectFactory.class);
            unmarshaller = context.createUnmarshaller();
        } catch (JAXBException jex) {
            throw new RuntimeException("Failed to create an instance of JAXB context", jex);
        }
        responseReader = new ResponseReaderRoutine(logger, integrationProperties, unmarshaller);
        logger.info("{} started", this.getClass());
    }

    @Override
    public LoyaltyResult doDiscount(Receipt receipt) {
        logger.info("doDiscount called");
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getLoyaltyPlugin() == null || pr.getLoyaltyPlugin().getDoDiscount() == null) {
            logger.info("doDiscount returns an empty result");
            return new LoyaltyResult();
        }
        displayRoutine.displayMessages(display, pr.getLoyaltyPlugin().getDoDiscount().getCustomerDisplayMessage());
        LoyaltyResult result = new LoyaltyResult();
        for(Discount d : pr.getLoyaltyPlugin().getDoDiscount().getDiscounts()) {
            result.getDiscounts().add(new ru.crystals.pos.api.ext.loyal.dto.Discount(
              d.getPosition(),
              d.getCardNumber(),
              d.getValue(),
              d.getId()
            ));
        }
        if(pr.getLoyaltyPlugin().getDoDiscount().getAccruals() != null) {
            BonusAccrualResult bar = new BonusAccrualResult(
                    pr.getLoyaltyPlugin().getDoDiscount().getAccruals().getTransactionId(),
                    pr.getLoyaltyPlugin().getDoDiscount().getAccruals().getAmount()
            );
            result.getBonusAccrualResults().put(pr.getLoyaltyPlugin().getDoDiscount().getAccruals().getCard(), bar);
        }
        if(pr.getLoyaltyPlugin().getDoDiscount().getTakenBonuses() != null) {
            result.getBonusesTakenIntoAccount().put(
                    pr.getLoyaltyPlugin().getDoDiscount().getTakenBonuses().getCard(),
                    pr.getLoyaltyPlugin().getDoDiscount().getTakenBonuses().getAmount()
            );
        }
        if(pr.getLoyaltyPlugin().getDoDiscount().getWriteOffLimits() != null) {
            result.getBonusWriteOffsLimits().put(
                    pr.getLoyaltyPlugin().getDoDiscount().getWriteOffLimits().getCard(),
                    pr.getLoyaltyPlugin().getDoDiscount().getWriteOffLimits().getAmount()
            );
        }
        for(AttributeEntry ae : pr.getLoyaltyPlugin().getDoDiscount().getReceiptAttributes()) {
            result.getExtendedReceiptAttributesMap().put(ae.getKey(), ae.getValue());
        }
        result.getMessagesToCashier().addAll(pr.getLoyaltyPlugin().getDoDiscount().getCashierMessage());
        for(PrintSlip ps : pr.getLoyaltyPlugin().getDoDiscount().getSlips()) {
            result.getPrintDocuments().add(SlipBuilder.createSlip(ps));
        }
        logger.info("doDiscount returns {}", result);
        return result;
    }

    @Override
    public DiscountCancellationResult cancelDiscount(Receipt receipt, LoyaltyResult results) {
        logger.info("cancelDiscount called");
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getLoyaltyPlugin() == null || pr.getLoyaltyPlugin().getCancelDiscount() == null) {
            logger.info("cancelDiscount returns an empty result");
            return null;
        }
        displayRoutine.displayMessages(display, pr.getLoyaltyPlugin().getCancelDiscount().getCustomerDisplayMessage());
        DiscountCancellationResult result = new DiscountCancellationResult();
        for(AttributeEntry ae : pr.getLoyaltyPlugin().getCancelDiscount().getReceiptAttributes()) {
            result.getExtendedReceiptAttributesMap().put(
                    ae.getKey(),
                    ae.getValue()
            );
        }
        logger.info("cancelDiscount returns {}", result);
        return result;
    }

    @Override
    public FiscalizationFeedback onReceiptFiscalized(Receipt receipt, LoyaltyResult loyaltyResult) {
        logger.info("onReceiptFiscalized called");
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getLoyaltyPlugin() == null || pr.getLoyaltyPlugin().getFiscalizationFeedback() == null) {
            logger.info("onReceiptFiscalized returns an empty result");
            return null;
        }
        displayRoutine.displayMessages(display, pr.getLoyaltyPlugin().getFiscalizationFeedback().getCustomerDisplayMessage());
        FiscalizationFeedback ff = new FiscalizationFeedback();
        for(PrintSlip ps : pr.getLoyaltyPlugin().getFiscalizationFeedback().getSlips()) {
            ff.getSlips().add(SlipBuilder.createSlip(ps));
        }
        if(pr.getLoyaltyPlugin().getFiscalizationFeedback().getAccruals() != null) {
            BonusAccrualResult bar = new BonusAccrualResult(
                    pr.getLoyaltyPlugin().getFiscalizationFeedback().getAccruals().getTransactionId(),
                    pr.getLoyaltyPlugin().getFiscalizationFeedback().getAccruals().getAmount()
            );
            ff.getBonusAccrualResults().put(pr.getLoyaltyPlugin().getFiscalizationFeedback().getAccruals().getCard(), bar);
        }
        logger.info("onReceiptFiscalized returns {}", ff);
        return ff;
    }

    @Override
    public PreFiscalizationFeedback eventBeforeReceiptFiscalized(Receipt receipt, LoyaltyResult loyResults) {
        logger.info("eventBeforeReceiptFiscalized called");
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getLoyaltyPlugin() == null || pr.getLoyaltyPlugin().getPrefiscalizationFeedback() == null) {
            logger.info("eventBeforeReceiptFiscalized returns an empty result");
            return null;
        }
        displayRoutine.displayMessages(display, pr.getLoyaltyPlugin().getPrefiscalizationFeedback().getCustomerDisplayMessage());
        FiscalizationFeedback ff = new FiscalizationFeedback();
        for(PrintSlip ps : pr.getLoyaltyPlugin().getPrefiscalizationFeedback().getSlips()) {
            ff.getSlips().add(SlipBuilder.createSlip(ps));
        }
        if(pr.getLoyaltyPlugin().getPrefiscalizationFeedback().getAccruals() != null) {
            BonusAccrualResult bar = new BonusAccrualResult(
                    pr.getLoyaltyPlugin().getPrefiscalizationFeedback().getAccruals().getTransactionId(),
                    pr.getLoyaltyPlugin().getPrefiscalizationFeedback().getAccruals().getAmount()
            );
            ff.getBonusAccrualResults().put(pr.getLoyaltyPlugin().getPrefiscalizationFeedback().getAccruals().getCard(), bar);
        }
        logger.info("eventBeforeReceiptFiscalized returns {}", ff);
        return ff;
    }

    @Override
    public Collection<LoyProviderFeedback> onSendFeedback(Collection<LoyProviderFeedback> feedback) {
        logger.info("onSendFeedback called {}", feedback);
        logger.info("onSendFeedback returns an empty result");
        return null;
    }

    @Override
    public PostDiscountResult onDiscountCalculationFinished(Receipt receipt) {
        logger.info("onDiscountCalculationFinished called");
        PluginResult pr = responseReader.getResultBag();
        if(pr == null || pr.getLoyaltyPlugin() == null || pr.getLoyaltyPlugin().getOnDiscountCalculationFinished() == null) {
            logger.info("onDiscountCalculationFinished returns an empty result");
            return null;
        }
        displayRoutine.displayMessages(display, pr.getLoyaltyPlugin().getOnDiscountCalculationFinished().getCustomerDisplayMessage());
        PostDiscountResult postDiscountResult = new PostDiscountResult();
        if(pr.getLoyaltyPlugin().getOnDiscountCalculationFinished().getWriteOffLimits() != null) {
            postDiscountResult.getBonusWriteOffLimits().put(
                    pr.getLoyaltyPlugin().getOnDiscountCalculationFinished().getWriteOffLimits().getCard(),
                    pr.getLoyaltyPlugin().getOnDiscountCalculationFinished().getWriteOffLimits().getAmount()
            );
        }
        logger.info("onDiscountCalculationFinished returns {}", postDiscountResult);
        return postDiscountResult;
    }
}
