package ru.crystals.pos.api.plugin;

import ru.crystals.pos.spi.plugin.FiscalizationListener;
import ru.crystals.pos.spi.plugin.ReconciliationReportMaker;
import ru.crystals.pos.spi.plugin.payment.CancelRequest;
import ru.crystals.pos.spi.plugin.payment.PaymentRequest;
import ru.crystals.pos.spi.plugin.payment.RefundRequest;

/**
 * Интерфейс плагина оплат.
 *
 */
public interface PaymentPlugin extends FiscalizationListener, ReconciliationReportMaker {

    /**
     * Запрос на выполнение оплаты.
     * <p>
     * Касса вызывает этот метод, когда необходимо выполнить оплату. Все
     * необходимые данные (<i>содержимое чека, сумма к оплате</i>) передаются в
     * аргументе {@code paymentRequest}.
     * <p>
     * Результат выполнения/не выполнения оплаты возвращается в кассу обратными
     * (<i>callback</i>) вызовами содержащимися в аргументе
     * {@code paymentRequest}.
     * 
     * @param paymentRequest
     *            необходимые для оплаты данные и реализация callback
     *            интерфейса.
     * 
     * @see ru.crystals.pos.spi.plugin.payment.PaymentRequest
     */
    void doPayment(PaymentRequest paymentRequest);

    /**
     * Запрос на отмену выполненной оплаты.
     * <p>
     * Касса вызывает этот метод, когда необходимо аннулировать чек, в котором
     * была выполнена оплата. Все необходимые данные (<i>данные выполненной
     * оплаты</i>) передаются в аргументе {@code cancelRequest}.
     * <p>
     * Результат выполнения/не выполнения оплаты возвращается в кассу обратными
     * (<i>callback</i>) вызовами содержащимися в аргументе
     * {@code cancelRequest}.
     * 
     * @param cancelRequest
     *            необходимые для отмены оплаты данные и реализация callback
     *            интерфейса.
     * 
     * @see ru.crystals.pos.spi.plugin.payment.CancelRequest
     */
    void doPaymentCancel(CancelRequest cancelRequest);

    /**
     * Запрос на возврат выполненной оплаты.
     * <p>
     * Касса вызывает этот метод, когда выполняется возврат чека (<i>целиком или
     * частично</i>) и необходимо вернуть деньги покупателю. Все необходимые
     * данные (<i>чек продажи, сумма к возврату</i>) передаются в аргументе
     * {@code refundRequest}.
     * <p>
     * Результат выполнения/не выполнения возврата возвращается в кассу
     * обратными (<i>callback</i>) вызовами содержащимися в аргументе
     * {@code refundRequest}.
     * 
     * @param refundRequest
     *            необходимые для возврата оплаты данные и реализация callback
     *            интерфейса.
     * 
     * @see ru.crystals.pos.spi.plugin.payment.RefundRequest
     */
    void doRefund(RefundRequest refundRequest);

    /**
     * Доступность плагина для оплаты.
     * <p> В данном методе можно выполнить проверку наличия необходимых настроек плагина, 
     * если они отсутствуют или некорректны, то можно запретить выбор данного способа оплаты. 
     * @return true - данный способ оплаты доступен к выбору<br>
     *         false - данный способ оплаты НЕ доступен к выбору
     */
    boolean isAvailable();
}
