package ru.crystals.pos.api.plugin.card;

import java.util.HashMap;
import java.util.Map;

/**
 * Сущность, описывающая отклик на запрос списания бонусов с карты.
 */
public class BonusWriteOffOperationResponse extends BonusOperationResponse {

    /**
     * Текст сообщения, что надо показать кассиру
     */
    private String operatorMessage;

    /**
     * Мапа расширенных атрибутов чека, которые плагин может к чеку прицепить.<br>
     * Необходима на случай, если вдруг плагинам нужно будет хранить какую-то исключительно свою информацию в чеке,
     * которая затем может выгружаться на стороне сервера в ERP или ещё зачем.
     */
    private Map<String, String> receiptExtendedAttributesMap;

    /**
     * Мапа расширенных атрибутов карты, которые плагин может к карте прицепить.<br>
     * Необходима на случай, если вдруг плагинам нужно будет хранить какую-то исключительно свою информацию в карте,
     * которая затем может выгружаться на стороне сервера в ERP или ещё зачем.
     */
    private Map<String, String> cardExtendedAttributesMap;

    /**
     * Конструкто класса. Создаёт новый экземпляр класса {@link BonusWriteOffOperationResponse}
     * @param responseStatus результат проведения операции списания
     * @param txId идентификатор транзакции списания
     * @param operatorMessage сообщение кассиру
     */
    public BonusWriteOffOperationResponse(BonusOperationStatus responseStatus, String txId, String operatorMessage) {
        super(responseStatus, txId);
        this.operatorMessage = operatorMessage;
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link BonusOperationResponse}
     * @see #BonusWriteOffOperationResponse(BonusOperationStatus, String, String)
     */
    public BonusWriteOffOperationResponse() {
        // default
    }

    /**
     * Вернет текст сообщения, что надо показать кассиру по результатам списания.
     * <p>
     * Например, данное сообщение может соджержать кратное локализованное описание ошибки, в понятном кассиру виде.
     * Или какой-нибудь рекламный текст (например, если это оказалось милионное успешное списание средств в процессинге).
     *
     * @return {@code null} либо пустая строка, если не надо показывать сообщений
     * NOTE: если вернет пустую строку, но {@link #getResponseStatus() результат} операции при этом будет отрицательным, то касса SET10 самомстоятельно
     * сформирует шаблонное сообщение об ошибке.
     */
    public String getOperatorMessage() {
        return operatorMessage;
    }

    /**
     * Устанавливает текст сообщения кассиру.
     * @param operatorMessage текст сообщения кассиру или null, если такого нет.
     */
    public void setOperatorMessage(String operatorMessage) {
        this.operatorMessage = operatorMessage;
    }

    /**
     * Мапа расширенных атрибутов, которыми плагин лояльности может наградить чек.<br>
     * Заполняется плагином в случае, если он хочет добавить к чеку какие-то дополнительные параметры.
     * @return мапу расширенных атрибутов чека или пустую мапу, если таких нет.
     */
    public Map<String, String> getReceiptExtendedAttributesMap() {
        if(receiptExtendedAttributesMap == null) {
            receiptExtendedAttributesMap = new HashMap<>();
        }
        return receiptExtendedAttributesMap;
    }

    /**
     * Мапа расширенных атрибутов карты, которые плагин может к карте прицепить.<br>
     * Необходима на случай, если вдруг плагинам нужно будет хранить какую-то исключительно свою информацию в карте,
     * которая затем может выгружаться на стороне сервера в ERP или ещё зачем.
     * @return мапу расширенных атрибутов карты или пустую мапу, если таких нет.
     */
    public Map<String, String> getCardExtendedAttributesMap() {
        if(cardExtendedAttributesMap == null) {
            cardExtendedAttributesMap = new HashMap<>();
        }
        return cardExtendedAttributesMap;
    }

    @Override
    public String toString() {
        return "BonusWriteOffOperationResponse{" +
                "operatorMessage='" + operatorMessage + '\'' +
                ", receiptExtendedAttributesMap=" + receiptExtendedAttributesMap +
                ", cardExtendedAttributesMap=" + cardExtendedAttributesMap +
                '}';
    }
}
