package ru.crystals.pos.api.plugin.card;

import ru.crystals.pos.api.comm.CommunicationMessage;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Сущность описывающая информацию по карте лояльности
 * */
public class CardInfo {

    /**
     * Статус поиска карты в процессинге
     * */
    private CardSearchResponseStatus cardSearchStatus;

    /**
     * Набор ключей и значений представляющих непосредственно информацию по карте
     * */
    private Map<String, String> info;

    /**
     * Заголовок окна
     * */
    private String title;

    /**
     * Список расширенных сообщений кассиру.
     * Используется для показа кассиру альтернативных сообщений.
     */
    private List<CommunicationMessage> communicationMessages = new ArrayList<>();

    /**
     * Флаг: разрешить печатать информацию по карте на ФР
     * @since 0.0.69
     */
    private boolean allowPrint;

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CardInfo}.
     * @param status статус ответа
     * @param info мапа, содержащая информацию по карте. Ключом в мапе служит человекочитаемое название поля, значением - человекочитаемое значение поля.
     *             Например: ключ: &quot;Держатель&quot;, значение: &quot;Иванов И.А&quot;
     */
    public CardInfo(CardSearchResponseStatus status, Map<String, String> info) {
        this.cardSearchStatus = status;
        this.info = info;
    }

    /**
     * Возврвщает статус запроса информации по карте.
     * @return статус запроса информации по карте.
     */
    public CardSearchResponseStatus getCardSearchStatus() {
        return cardSearchStatus;
    }

    /**
     * Устанавливает статус запроса информации по карте.
     * @param cardSearchStatus статус запроса информации по карте.
     */
    public void setCardSearchStatus(CardSearchResponseStatus cardSearchStatus) {
        this.cardSearchStatus = cardSearchStatus;
    }

    /**
     * Возвращает заголовок, которым следует озаглавить окно с информацией по карте.
     * @return заголовок, которым следует озаглавить окно с информацией по карте.
     */
    public String getTitle() {
        return title;
    }

    /**
     * Устанавливает заголовок, которым следует озаглавить окно с информацией по карте.
     * @param title заголовок, которым следует озаглавить окно с информацией по карте.
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * Возвращает мапу информаци по карте. Ключом в мапе является человекочитаемое название поля, значением - значение поле.
     * Пример: ключ: &quot;Номер карты&quot;, значение: &quot;123456789&quot;
     * @return мапа информации по карте или пустая мапа, если такой нет.
     */
    public Map<String, String> getInfo() {
        if(info == null) {
            info = new TreeMap<>();
        }
        return info;
    }

    /**
     * Устанавливает мапу информации по карте. Ключом в мапе является человекочитаемое название поля, значением - значение поле.
     * Пример: ключ: &quot;Номер карты&quot;, значение: &quot;123456789&quot;
     * @param info мапа информации по карте.
     */
    public void setInfo(Map<String, String> info) {
        this.info = info;
    }

    /**
     * Возвращают список "расширенных" сообщений кассиру, в которых допустимо управлять иконкой, а также делить
     * текст на смысловые блоки. Сюда можно добавить своих сообщений кассиру.
     * @return список расширенных сообщений кассиру, куда можно добавить своё сообщение, или пустой список, если таких нет.
     */
    public List<CommunicationMessage> getCommunicationMessages() {
        return communicationMessages;
    }

    public boolean isAllowPrint() {
        return allowPrint;
    }

    public void setAllowPrint(boolean allowPrint) {
        this.allowPrint = allowPrint;
    }

    @Override
    public String toString() {
        return "CardInfo{" +
                "cardSearchStatus=" + cardSearchStatus +
                ", info=" + info +
                ", title='" + title + '\'' +
                ", allowPrint='" + allowPrint + '\'' +
                '}';
    }
}
