package ru.crystals.pos.api.plugin.goods;

import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.spi.receipt.Merchandise;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Сущность для передачи кассе данных о новой позиции чека
 */
public class NewLineItem {

    private String marking;

    private String name;

    private String barcode;

    private BigDecimal price;

    private long quantity;

    private List<Slip> slips;

    private Map<String, String> data;

    private float nds = 0.f;

    /**
     * Конструктор сущности с обязательными полями.
     *
     * @param marking артикул товара (максимум 30 символов)
     * @param name наименование (максимум 255 символов)
     * @param price цена (&gt;= 0, scale = 2)
     * @param quantity количество в тысячных долях единиц (т.е. 1000 == 1 шт. (или 1 кг.), 100 == 100 гр.)
     * @param nds ставка НДС в процентах (пример: 0.f, 10.f, 18.f)
     */
    public NewLineItem(String marking, String name, BigDecimal price, long quantity, float nds) {
        this.marking = marking;
        this.name = name;
        this.price = price;
        this.quantity = quantity;
        this.nds = nds;
    }

    /**
     * Конструктор сущности на основании товара.<p>
     * Количество устанавливается равным 1 шт.
     *
     * @param merchandise товар {@link Merchandise}
     */
    public NewLineItem(Merchandise merchandise) {
        this.marking = merchandise.getMarking();
        this.name = merchandise.getName();
        this.price = merchandise.getPrice();
        this.quantity = 1000;
        this.barcode = merchandise.getBarcode();
        this.nds = merchandise.getNds();

    }

    /**
     * Вернет артикул позиции
     * @return артикул
     */
    public String getMarking() {
        return marking;
    }

    /**
     * Присвоить артикул позиции (максимум 30 символов)
     * @param marking артикул
     */
    public void setMarking(String marking) {
        this.marking = marking;
    }

    /**
     * Вернет наименование позиции
     * @return наименование позиции
     */
    public String getName() {
        return name;
    }

    /**
     * Присвоить наименование позиции (максимум 255 символов)
     * @param name наименование позиции
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Вернет штрихкод позиции
     * @return штрихкод
     */
    public String getBarcode() {
        return barcode;
    }

    /**
     * Присвоить штрихкод позиции (максимум 30 символов)
     * @param barcode штрихкод
     */
    public void setBarcode(String barcode) {
        this.barcode = barcode;
    }

    /**
     * Вернет цену позиции
     * @return цена
     */
    public BigDecimal getPrice() {
        return price;
    }

    /**
     * Присвоить цену позиции
     * @param price цена (&gt;= 0, scale = 2)
     */
    public void setPrice(BigDecimal price) {
        this.price = price;
    }

    /**
     * Вернет количество товара в позиции
     * @return количество в тысячных долях единиц (т.е. 1000 == 1 шт. (или 1 кг.), 100 == 100 гр.)
     */
    public long getQuantity() {
        return quantity;
    }

    /**
     * Присвоить количество товара в позиции
     * @param quantity количество в тысячных долях единиц (т.е. 1000 == 1 шт. (или 1 кг.), 100 == 100 гр.)
     */
    public void setQuantity(long quantity) {
        this.quantity = quantity;
    }

    /**
     * Печатные документы, прикрепленные к данной позиции
     * @return список печатных документов, прикрепленных к данной позиции или пустой список, если таких нет.
     */
    public List<Slip> getSlips() {
        if (slips == null) {
            slips = new ArrayList<>();
        }
        return slips;
    }

    /**
     * Дополнительные данные товарной позиции {@code <ключ, значение>}.<br>
     * @return коллекция ключей и значений
     */
    public Map<String, String> getData() {
        if (data == null) {
            data = new HashMap<>();
        }
        return data;
    }

    /**
     * Вернет ставку НДС в процентах
     * @return значение НДС
     */
    public float getNds() {
        return nds;
    }

    /**
     * Установить ставку НДС в процентах
     * @param nds ставка (пример: 0.f, 10.f, 18.f)
     */
    public void setNds(float nds) {
        this.nds = nds;
    }

    @Override
    public String toString() {
        return "NewLineItem{" +
                "marking='" + marking + '\'' +
                ", name='" + name + '\'' +
                ", barcode='" + barcode + '\'' +
                ", price=" + price +
                ", quantity=" + quantity +
                ", slips=" + slips +
                ", data=" + data +
                ", nds=" + nds +
                '}';
    }
}
