package ru.crystals.pos.api.plugin.payment;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Данные выполненной оплаты.
 * <p>
 * Данный класс используется для обмена информацией с кассой о результатах выполнения оплаты.<br>
 * Касса сохраняет поля этого класса в БД.<br>
 * Поле {@code slip} используется для печати дополнительного бумажного квитка после фискализации чека.
 * <p>
 * Эти данные необходимы:<br>
 * - при аннулировании или возврате чека;<br>
 * - для экспорта данных оплат чека в ERP системы сети магазинов;<br>
 * - при визуализации чека на сервере Set Retail 10.
 * <p>
 * Информация о том, какие поля используются для визуализации и экспорта указываются в манифесте: <b>metainf.xml</b>
 */
public class Payment {

    /**
     * Сумма оплаты
     */
    private BigDecimal sum;

    /**
     * Данные транзакции выполненной оплаты для сохранения в БД &lt;ключ, значение&gt;
     */
    private Map<String, String> data = new HashMap<>();

    /**
     * Слипы для печати
     */
    private List<String> slips;

    /**
     * Дополнительные данные в формате &lt;ключ, значение&gt;, которые будут добавлены в чек
     */
    private Map<String, String> extendedReceiptAttributes = new HashMap<>();

    /**
     * Признак возврата оплаты наличными.
     */
    private boolean refundPaymentAsCash = false;

    /**
     * Признак возврата оплаты банковской картой.
     */
    private boolean refundPaymentAsBankCard = false;

    /**
     * Сумма оплаты
     * 
     * @return сумма с двумя знаками после запятой (scale = 2)
     */
    public BigDecimal getSum() {
        return sum;
    }

    /**
     * Сумма оплаты
     * 
     * @param sum сумма с двумя знаками после запятой (scale = 2)
     */
    public void setSum(BigDecimal sum) {
        this.sum = sum;
    }

    /**
     * Данные транзакции выполненной оплаты для сохранения в БД {@code<ключ, значение>}
     * 
     * @return коллекция ключей и значений
     */
    public Map<String, String> getData() {
        return data;
    }

    /**
     * Получить значение поля по ключу из сохраненной в БД транзакции.
     * 
     * @param key ключ, по которому ищется значение
     * @return значение поля или null, если значение отсутствует
     * @see #getData()
     */
    public String getData(String key) {
        return data.get(key);
    }
    
    /**
     * Слип для печати
     * @deprecated метод устарел и не рекомендуется к использованию. Используйте метод {@link #getSlips()} для
     * получения и добавления в чек новых слипов.
     * @return текст слипа
     */
    @Deprecated
    public String getSlip() {
        if (slips == null || slips.isEmpty()) {
            return null;
        }
        return slips.get(0);
    }

    /**
     * Слип для печати
     * @deprecated метод устарел и не рекомендуется к использованию. Используйте метод {@link #getSlips()} для
     * получения и добавления в список слипов нового слипа.
     * @param slip текст слипа
     */
    @Deprecated
    public void setSlip(String slip) {
        if (this.slips == null) {
            this.slips = new LinkedList<>();
        } else {
            this.slips.clear();
        }
        this.slips.add(slip);
    }

    /**
     * Слипы для печати
     *
     * @return slips слипы
     */
    public List<String> getSlips() {
        if (slips == null) {
            slips = new LinkedList<>();
        }
        return slips;
    }
    
    /**
     * Слипы для печати
     *
     * @param slips слипы
     */
    public void setSlips(List<String> slips) {
        this.slips = slips;
    }

    /**
     * Дополнительные данные в формате &lt;ключ, значение&gt;, которые будут добавлены в чек
     *
     * @return коллекция ключей и значений
     *
     * @since 0.0.71
     */
    public Map<String, String> getExtendedReceiptAttributes() {
        return extendedReceiptAttributes;
    }

    /**
     * Признак возврата оплаты наличными.<br>
     * Позволяет вернуть оплату наличными при невозможности возврата плагином оплат (например, при отсутствии связи).
     *
     * @return признак возврата оплаты наличными.
     *
     * @apiNote Реализовано только для {@link ru.crystals.pos.spi.plugin.payment.PaymentCallback#paymentNotCompleted(ru.crystals.pos.api.plugin.payment.Payment)}
     * @since 0.0.72
     */
    public boolean isRefundPaymentAsCash() {
        return refundPaymentAsCash;
    }

    /**
     * Признак возврата оплаты банковской картой.<br>
     * Позволяет вернуть оплату банковской картой при невозможности возврата плагином оплат (например, при отсутствии связи).
     *
     * @return признак возврата оплаты банковской картой.
     *
     * @apiNote Реализовано только для {@link ru.crystals.pos.spi.plugin.payment.PaymentCallback#paymentNotCompleted(ru.crystals.pos.api.plugin.payment.Payment)}
     * @since 0.0.90
     */
    public boolean isRefundPaymentAsBankCard() {
        return refundPaymentAsBankCard;
    }

    /**
     * Устанавливает признак возврата оплаты наличными.<br>
     * Позволяет вернуть оплату наличными при невозможности возврата плагином оплат (например, при отсутствии связи).
     *
     * @param refundPaymentAsCash устанавливаемое значение
     *
     * @apiNote Реализовано только для {@link ru.crystals.pos.spi.plugin.payment.PaymentCallback#paymentNotCompleted(ru.crystals.pos.api.plugin.payment.Payment)}
     * @since 0.0.72
     */
    public void setRefundPaymentAsCash(boolean refundPaymentAsCash) {
        this.refundPaymentAsCash = refundPaymentAsCash;
    }

    /**
     * Устанавливает признак возврата оплаты наличнымибанковской картой.<br>
     * Позволяет вернуть оплату банковской картой при невозможности возврата плагином оплат (например, при отсутствии связи).
     *
     * @param refundPaymentAsBankCard устанавливаемое значение
     *
     * @apiNote Реализовано только для {@link ru.crystals.pos.spi.plugin.payment.PaymentCallback#paymentNotCompleted(ru.crystals.pos.api.plugin.payment.Payment)}
     * @since 0.0.90
     */
    public void setRefundPaymentAsBankCard(boolean refundPaymentAsBankCard) {
        this.refundPaymentAsBankCard = refundPaymentAsBankCard;
    }

    @Override
    public String toString() {
        return "Payment{" +
                "sum=" + sum +
                ", data=" + data +
                ", slips=" + slips +
                ", extendedReceiptAttributes=" + extendedReceiptAttributes +
                ", refundPaymentAsCash=" + refundPaymentAsCash +
                ", refundPaymentAsBankCard=" + refundPaymentAsBankCard +
                '}';
    }
}
