package ru.crystals.pos.spi;

import ru.crystals.pos.api.plugin.user.User;

/**
 * Интерфейс для получения информации о кассе.
 *
 **/
public interface POSInfo extends Injectable {

    /**
     * Номер магазина в котором работает касса.
     *
     * @return номер магазина в диапазоне [1..99999]
     */
    int getShopNumber();

    /**
     * Номер кассы в магазине.
     *
     * @return номер кассы в диапазоне [1..999]
     */
    int getPOSNumber();

    /**
     * Получает номер текущей смены.
     * @return номер текущей смены.
     */
    int getShiftNumber();

    /**
     * Получить наименование магазина.
     *
     * @return наименование магазина
     */
    String getShopName();

    /**
     * Получить информацию о текущем пользователе
     * @return данные о пользователе
     */
    User getUser();

    /**
     * Получить информацию о пользователе по паролю
     *
     * @return данные о пользователе
     */
    User getUser(String password);

    /**
     * Получить тип кассы.
     * @return тип кассы.
     * @since 0.0.46
     */
    POSType getPOSType();

    /**
     * Возвращает идентификатор шаблона кассы.<br>
     * Под шаблоном кассы понимается касса с выделенным функционалом - касса продажи алкоголя, касса возвратов, касса в линейке, etc.
     * Шаблоны касс заводятся и настраиваются пользователем, посему могут иметь любое название и конфигурацию. Однака каждый такой шаблон
     * имеет свой уникальный идентификатор, который и возвращается этим методом.
     * @return идентификатор шаблона кассы.
     * @since 0.0.46
     */
    long getPOSTemplateGuid();

    /**
     * Возвращает шаблон кассы.
     * @return шаблон кассы
     * @since 0.0.96
     */
    POSTemplate getPOSTemplate();

    /**
     * Возвращает текущий режим работы кассы. Под режимом понимается продажа товара, возврат по чеку, произвольный возврат, etc.
     * @return режим работы кассы.
     * @since 0.0.46
     */
    PosOperationMode getOperationMode();

    /**
     * Возвращает сумму наличных в денежном ящике в минимальном номинале (для РФ - копейка).
     * @return сумма наличных (по умолчанию = -1, если получение суммы недоступно)
     * @since 0.0.65
     */
    default long getCashAmount() {
        return -1L;
    }

    /**
     * Возвращает ИНН магазина
     * @return ИНН магазина
     */
    default String getShopINN() {
        return null;
    }

    /**
     * Возвращает КПП магазина
     * @return КПП магазина
     * @since 0.0.66
     */
    default String getShopKPP() {
        return null;
    }

    /**
     * Возвращает адрес магазина
     *
     * @return адрес магазина
     * @since 0.0.85
     */
    default String getShopAddress() {
        return null;
    }

    /**
     * Возвращает имя юридического лица
     *
     * @return имя юридического лица
     * @since 0.0.95
     */
    default String getShopLegalName() {
        return null;
    }

    /**
     * Возвращает интерфейс для получения данных из ФР
     *
     * @return интерфейс для получения данных из ФР
     * @since 0.0.98
     */
    default FiscalInfo getFiscalInfo() {
        return null;
    }

    /**
     * Возвращает имя принтера, подключенного к кассе
     *
     * @return имя принтера
     * @since 0.0.99
     */
    default String getPrinterName() {
        return null;
    }

    /**
     * Признак разрешения возврата чека только в рамках текущего календарного дня.
     * Признак true, то чек можно вернуть только если продажа была в текущем календарном дне.
     *
     * @return true / false
     * @since 0.0.102
     */
    default boolean isRefundReceiptDayToDay() {
        return false;
    }
}
