package ru.crystals.pos.spi.plugin.goods;

import ru.crystals.pos.api.ext.loyal.dto.AddForSaleResult;
import ru.crystals.pos.api.plugin.goods.NewLineItem;
import ru.crystals.pos.spi.Action;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;

/**
 * Интерфейс для оповещения кассы о результатах выполнения запроса на добавление товара в чек продажи.
 * Через этот слушатель плагин типа товаров оповещает о завершении создания позиции.
 */
public interface AddForSaleCallback {

    /**
     * Результат: товар может быть добавлен в чек продажи.
     *
     * @param newLineItem данные, которые необходимо сохранить в БД
     * @throws InvalidLineItemException исключение возникает, если аргумент {@code newLineItem} содержит некорректные данные.<br>
     * <b>Данные не будут сохранены в БД кассы.</b><br>
     * Локализованное сообщение можно получить методом {@link InvalidLineItemException#getLocalizedMessage()}<br>
     * <pre>Причинами возникновения исключения могут быть:
     *  - некорректно заполненные или пустые требуемые поля;
     *  - другими настройками кассы, блокирующими добавление (например превышение суммы чека).
     * </pre>
     * В данной ситуации рекомендуется отобразить сообщение об ошибке и,<br>если возможно, отменить добавление товара.
     *
     * @deprecated Метод устарел и не рекомендуется к использованию. Используйте {@link #completed(AddForSaleResult)}
     * @throws InvalidLineItemException если товар не может быть добавлен по причине ошибки.
     */
    @Deprecated
    default void completed(NewLineItem newLineItem) throws InvalidLineItemException {
        completed(new AddForSaleResult(newLineItem));
    }

    /**
     * Добавление товара в чек
     * @param newLineItem данные, которые необходимо сохранить в БД
     * @param actionsToTake действия, которые нужно произвести, если добавление пройдет успешно
     * @throws InvalidLineItemException если данная позиция не может быть добавлена.
     * @apiNote
     * Важно знать:<br>
     * 1. Каждый Action атомарен и не влияет на выполнение других<br>
     * 2. Выполнение не гарантируется и не влияет на добавление товара в чек
     * */
    default void completed(NewLineItem newLineItem, Collection<Action> actionsToTake) throws InvalidLineItemException {
        AddForSaleResult result = new AddForSaleResult(newLineItem);
        if (actionsToTake != null && !actionsToTake.isEmpty()) {
            result.getActionsToTake().addAll(actionsToTake);
        }
        completed(result);
    }

    /**
     * Добавление товара в чек.
     *
     * @param result результат обработки плагином события добавления товара в чек
     * @throws InvalidLineItemException если данная позиция не может быть добавлена
     *
     * @since 0.0.71
     */
    void completed(AddForSaleResult result) throws InvalidLineItemException;

    /**
     * Результат: товар не может быть добавлен в чек продажи.<p>
     * Причины могут быть разными:<br>
     * - покупатель отказался;<br>
     * - не доступен процессинг;<br>
     * - не действительный товар;<br>
     * - и т.д.
     * <p>
     * @throws CancelDeniedException возникает, когда у кассира нет привилегии для отмены позиции.<br>
     * Управление остается у плагина.<br>
     * Решается административно, например авторизацией пользователем имеющим данную привилегию.
     */
    default void notCompleted() throws CancelDeniedException {
        notCompleted(new AddForSaleResult());
    }

    /**
     * Результат: товар не может быть добавлен в чек продажи.<p>
     * Причины могут быть разными:<br>
     * - покупатель отказался;<br>
     * - не доступен процессинг;<br>
     * - не действительный товар;<br>
     * - и т.д.
     * <p>
     * @param result результат обработки плагином события добавления товара в чек
     * @throws CancelDeniedException возникает, когда у кассира нет привилегии для отмены позиции.<br>
     * Управление остается у плагина.<br>
     * Решается административно, например авторизацией пользователем имеющим данную привилегию.
     *
     * @since 0.0.71
     */
    void notCompleted(AddForSaleResult result) throws CancelDeniedException;
}
