package ru.crystals.pos.spi.receipt;

import ru.crystals.pos.api.ext.loyal.dto.Slip;
import ru.crystals.pos.api.receipt.InsertType;
import ru.crystals.pos.spi.MarkedProductType;
import ru.crystals.pos.spi.annotation.POSPlugin;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * Сущность, описывающая позицию в чеке.
 * <p>
 * NOTE: все денежные суммы представлены в {@link BigDecimal} с двумя знаками после запятой, т.е. scale=2
 *
 * @author aperevozchikov
 */
public interface LineItem {

    /**
     * Вернет общий размер скидок на данную позицию
     *
     * @return не {@code null}
     */
    BigDecimal getDiscountValue();

    /**
     * Вернет Товар, что продается в этой позиции
     *
     * @return не {@code null}
     */
    Merchandise getMerchandise();

    /**
     * Вернет Порядковый номер позиции в чеке
     *
     * @return Порядковый номер позиции в чеке
     */
    int getNumber();

    /**
     * Вернет Флаг-признак, указывающий можно ли давать скидку на данную позицию.
     * Учитывает соответствующий флаг для товара, а также дополнительные признаки для конкретной позиции
     * (например, переданные из мягкого чека).
     *
     * @return {@code false}, если скидку на данную позицию давать нельзя
     */
    boolean isDiscountable();

    /**
     * Вернет Оригинальную ЦЕНУ (стоимость за единицу товара в позиции) (не будет переписываться при расчёте скидок)
     *
     * @return ЦЕНУ до расчета скидок
     */
    BigDecimal getOriginalPrice();

    /**
     * Вернет Количество товара в позиции, в тысячных долях единиц СИ (т.е. 1000 == 1 шт. (или 1 кг.), 100 == 100 гр.)
     *
     * @return количество товара в позиции, в "граммах"; строго положительное число
     */
    long getQuantity();

    /**
     * Вернет СТОИМОСТЬ позиции с учетом уже данных скидок
     *
     * @return не {@code null}
     */
    BigDecimal getSum();

    /**
     * Вернет минимально допустимую ЦЕНУ товара в данной позиции
     *
     * @return не {@code null}; не отрицательное число
     */
    BigDecimal getMinPrice();

    /**
     * Печатные документы, прикрепленные к данной позиции.
     * @return список печатных документов, прикрепленных к данной позиции или пустой список, если таких нет.
     */
    default List<Slip> getSlips() {
        return new ArrayList<>();
    }

    /**
     * Дополнительные данные товарной позиции {@code <ключ, значение>}.<br>
     * @return коллекция ключей и значений
     */
    default Map<String, String> getData() {
        return new HashMap<>();
    }

    /**
     * Вернет id типа товара (плагина) создавшего эту позицию чека.
     * @return строковый идентификатор типа товара
     * @see POSPlugin#id()
     */
    default String getPluginId() {
        return "";
    }

    /**
     * Вернет флаг-признак, указывающий можно ли давать скидку бонусами на данную позицию.
     * <p>
     * NOTE: если скидки на данную позицию запрещены (см. {@link #isDiscountable()}), то значение данного признака
     * следует игнорировать: на позицию нельзя давать никакие скидки - в том числе бонусами.
     * </p>
     *
     * @return {@code false}, если нельзя использовать бонусы как скидку для данной позиции
     * @since 0.0.27
     */
    default boolean isPayBonusAllowed() {
        return true;
    }

    /**
     * Вернет флаг-признак, указывающий можно ли начислять бонусы за приобретение данной позиции.
     *
     * @return {@code false}, если нельзя начислять бонусы за данную позицию
     * @since 0.0.27
     */
    default boolean isAccrueBonusAllowed() {
        return true;
    }

    /**
     * Возвращает коллекцию скидок, которые были применены на данную позицию.<br>
     * Данная коллекция содержит детализированную информацю о предоставленных на позицию скидках.
     * @return коллекция предоставленных на позицию скидок или пустая коллекция, если таких нет.
     * @since 0.0.46
     */
    default Collection<AppliedDiscount> getAppliedDiscounts() {
        return Collections.emptyList();
    }

    /**
     * Возвращает идентификатор мягкого чека из которого добавлена это позиция. <br>
     * @return не {@code null}
     * @since 0.0.48
     */
    default Optional<String> getSoftReceiptId() { return Optional.empty(); }

    /**
     * Возвращает позиционные купоны применённые к этой позиции
     * @return позиционные купоны применённые к этой позиции
     * @since 0.0.54
     */
    default Collection<Card> getPositionCoupons() {
        return Collections.emptyList();
    }

    /**
     * Возвращает коллекцию бонусов, которые были начислены на данную позицию.
     * @return коллекция начисленых на позицию бонусов.
     * @since 0.0.60
     */
    default Collection<AppliedBonusAmount> getAccruedBonuses() {
        return Collections.emptyList();
    }

    /**
     * Возвращает акцизную марку неалкогольной позиции, или null, при её отсутствии
     * Для алкоголя следует использовать {@link Receipt#getExciseBottles()}
     * @return акцизная марка
     * @since 0.0.63
     */
    default String getExcise() {
        return null;
    }

    /**
     * Возвращает тип маркировки или null, если товар не маркированный
     * @return тип маркировки
     * @see MarkedProductType
     * @since 0.0.63
     */
    default MarkedProductType getMarkedProductType() {
        return null;
    }

    /**
     * GUID рекламной акции, по которой данная позиция добавлена в чек
     * @return GUID РА
     * @since 0.0.68
     */
    default Long getAdvertisingActionGuid() {
        return null;
    }

    /**
     * Способ добавления позиции.
     */
    default Optional<InsertType> getInsertType() {
        return Optional.empty();
    }

    /**
     * Мерность товара
     *
     * @return Мерность товара
     * @since 0.0.85
     */
    default BigDecimal getPrecision() {
        return null;
    }

    /**
     * Уникальный код единицы измерения товара
     *
     * @since 0.0.99
     */
    default String getMeasureCode() {
        return null;
    }

    /**
     * Наименование единицы измерения товара
     *
     * @since 0.0.99
     */
    default String getMeasureName() {
        return null;
    }

    /**
     * Код единицы измерения по ОКЕИ (Общероссийский классификатор единиц измерения)
     *
     * @since 0.0.102
     */
    default String getMeasureRcumCode() {
        return null;
    }

    /**
     * Сумма налога товара в копейках
     *
     * @return сумма налога
     */
    default BigDecimal getVatSum() {
        return null;
    }

    /**
     * Номер позиции в оригинальном чеке (чек продажи)
     */
    default Integer getNumberInOriginal() {
        return null;
    }

    /**
     * Признак способа расчёта.
     *
     * @return признак способа расчёта.
     * @since 0.0.96
     */
    default Integer getCalculationMethod() {
        return null;
    }

    /**
     * Номер ГТД позиции
     *
     * @since 0.0.99
     */
    default String getGtdNumber() {
        return null;
    }

    /**
     * Код страны происхождения товара
     *
     * @since 0.0.99
     */
    default String getCountryCode() {
        return null;
    }

    /**
     * Признак предмета расчета
     *
     * @since 0.0.99
     */
    default Integer getCalculationSubject() {
        return null;
    }

    /**
     * Данные маркировки
     *
     * @since 0.0.102
     */
    default MarkInfo getMarkInfo() {
        return null;
    }

    /**
     * Признак агента по предмету расчета
     *
     * @since 0.0.102
     */
    default AgentSubject getAgentSubject() {
        return null;
    }

    /**
     * @return возвращает информацию о юридическом лице поставщика
     * @since 0.0.102
     */
    default JuristicProductSupplier getProductSupplier() {
        return null;
    }

}
