package ru.crystals.pos.spi.receipt;

import ru.crystals.pos.spi.GoodsType;
import ru.crystals.pos.spi.POSTemplate;

import java.math.BigDecimal;
import java.util.Optional;
import java.util.Collections;
import java.util.Map;

/**
 * Сущность, описывающая товар (в чековой позиции).
 *
 * @author aperevozchikov
 */
public interface Merchandise {

    /**
     * Вернет Артикул (уникальный идентификатор) товара
     *
     * @return артикул товара
     */
    String getMarking();

    /**
     * Вернет Название товара
     *
     * @return Название товара
     */
    String getName();

    /**
     * Вернет Код производителя
     *
     * @return Код производителя
     */
    String getManufacturer();

    /**
     * Вернет Код страны-производителя
     *
     * @return Код страны-производителя
     */
    String getCountry();

    /**
     * Вернет цену товара
     *
     * @return цена товара
     */
    BigDecimal getPrice();

    /**
     * Вернет штрихкод товара
     * @return штрихкод
     */
    String getBarcode();

    /**
     * Вернет ставку НДС в процентах
     * @return значение НДС
     */
    float getNds();

    /**
     * Возвращает тип товара.
     * @return тип товара.
     * @since 0.0.46
     */
    default GoodsType getGoodsType() { return GoodsType.PIECE; }

    /**
     * Возвращает идентификатор товарной группы, которой принадлежит товар.<br>
     * Товарная группа это набор товаров, объединенных каким-то признаком.
     * @return идентификатор товрной группы, которой прнадлежит товар или null если такого нет. Это идентификатор из справочника товарных групп SetRetail10.
     * @since 0.0.46
     */
    String getGroupCode();

    /**
     * Возвращает признак предмета расчёта товара.
     * @return признак предмета расчёта.
     * @since 0.0.77
     */
    Integer getCalculationSubject();

    /**
     * @return Признак "Объёмно-сортовой учёт"
     * @since 0.102
     */
    boolean isVolumeVarietalCounting();

    /**
     * @return товарную группу, которой принадлежит товар или null если такой нет.
     * В отличии от {@link #getGroupCode()} может возвращать расширенную информацию о группе (имя, родительскую группу)
     * @since 0.0.73
     */
    default Group getGroup() {
        return () -> getGroupCode();
    }

    /**
     * Вернет флаг-признак, указывающий можно ли давать скидку на данный товар.
     *
     * @return {@code false}, если запрещен расчет скидок на данный товар
     * @since 0.0.68
     */
    default boolean isDiscountable() {
        return true;
    }

    /**
     * Получим доп.данные для плагина (например JSON, XML, строка с разделителем и т.п.). Как распарсить должен знать плагин.
     *
     * @return доп данные в виде строки
     * @since 0.0.79
     */
    default Optional<String> getExternalData() {
        return Optional.empty();
    }

    /**
     * Вернет флаг-признак запрета продажи товара юридическому лицу.
     *
     * @see POSTemplate#areAllProductTypesForJuristicPersonAllowed()
     * @return {@code true}, если запрещено продавать данный товар юридическому лицу
     * @since 0.0.96
     */
    default boolean isForbidSaleJuristicPerson() {
        return false;
    }

    /**
     * Вернет дополнительные свойства товара (например, hasDiscount флаг)
     * @return доп данные в виде ключ-значение
     * @since 0.0.x
     */
    default Map<String, String> getAdditionalProperties() {
        return Collections.emptyMap();
    }

    /**
     * Вернет штрих-код, использованный ШК-трансформером (если трансформер использовался)
     * @return значение ШК
     * @since 0.0.x
     */
    default Optional<String> getTransformerBarcode() {
        return Optional.empty();
    }
}
