package ru.crystals.pos.spi.ui;

import java.util.List;

/**
 * Набор параметров для открытия формы с таблицей, поддерживающей выбор строк.
 */
public class TableSelectionFormParameters {

    /**
     * Возвращает построителя конфигурации формы с таблицей, поддерживающей выбор строк.
     *
     * @return построитель конфигурации формы с таблицей, поддерживающей выбор строк
     */
    public static TableSelectionFormParameters.Builder builder() {
        return new TableSelectionFormParameters.Builder();
    }

    /**
     * Заголовок формы.
     */
    private final String title;

    /**
     * Дополнительный текст для отображения на форме.
     */
    private final String text;

    /**
     * Описание столбцов таблицы.
     */
    private final List<TableSelectionFormColumn> columns;

    /**
     * Описание строк таблицы:
     * Каждая Map в списке - отдельная строка. Ключи каждого элемента Map должны быть из ключей, заданных в columns
     *
     * @see TableFormColumn#getKey()
     */
    private final List<TableSelectionFormRow> rows;

    /**
     * Индекс текущей строки. Если строка не попадает в видимую область, то таблица будет отскроллирована.
     */
    private final Integer selectedRowIndex;

    /**
     * Описание действий на форме.
     */
    private final List<TableSelectionFormAction> actions;

    /**
     * Идентификатор действия (кнопки на POS-кассе), которое должно быть выбрано при открытии формы.
     */
    private final String selectedActionId;

    /**
     * Конструктор.
     *
     * @param title            заголовок формы
     * @param text             дополнительный текст для отображения на форме
     * @param columns          описание столбцов таблицы
     * @param rows             описание строк таблицы
     * @param selectedRowIndex индекс строки, которая должна быть выбрана
     * @param actions          описание действий на форме
     * @param selectedActionId идентификатор действия (кнопки на POS-кассе), которое должно быть выбрано при открытии формы
     */
    public TableSelectionFormParameters(String title, String text, List<TableSelectionFormColumn> columns,
                                        List<TableSelectionFormRow> rows, Integer selectedRowIndex,
                                        List<TableSelectionFormAction> actions, String selectedActionId) {
        this.title = title;
        this.text = text;
        this.columns = columns;
        this.rows = rows;
        this.selectedRowIndex = selectedRowIndex;
        this.actions = actions;
        this.selectedActionId = selectedActionId;
    }

    public String getTitle() {
        return title;
    }

    public String getText() {
        return text;
    }

    public List<TableSelectionFormColumn> getColumns() {
        return columns;
    }

    public List<TableSelectionFormRow> getRows() {
        return rows;
    }

    public Integer getSelectedRowIndex() {
        return selectedRowIndex;
    }

    public List<TableSelectionFormAction> getActions() {
        return actions;
    }

    public String getSelectedActionId() {
        return selectedActionId;
    }

    /**
     * Построитель конфигурации формы с таблицей, поддерживающей выбор строк.
     */
    public static class Builder {
        private String title;
        private String text;
        private List<TableSelectionFormColumn> columns;
        private List<TableSelectionFormRow> rows;
        private Integer selectedRowIndex;
        private List<TableSelectionFormAction> actions;
        private String selectedActionId;

        /**
         * Устанавливает заголовок формы.
         *
         * @param title заголовок формы
         * @return построитель
         */
        public Builder setTitle(String title) {
            this.title = title;
            return this;
        }

        /**
         * Устанавливает дополнительный текст для отображения на форме.
         *
         * @param text дополнительный текст для отображения на форме
         * @return построитель
         */
        public Builder setText(String text) {
            this.text = text;
            return this;
        }

        /**
         * Устанавливает описание столбцов таблицы.
         *
         * @param columns описание столбцов таблицы.
         * @return построитель
         */
        public Builder setColumns(List<TableSelectionFormColumn> columns) {
            this.columns = columns;
            return this;
        }

        /**
         * Устанавливает описание строк таблицы.
         *
         * @param rows описание строк таблицы
         * @return построитель
         */
        public Builder setRows(List<TableSelectionFormRow> rows) {
            this.rows = rows;
            return this;
        }

        /**
         * Устанавливает индекс строки, которая должна быть выбрана.
         *
         * @param selectedRowIndex индекс строки, которая должна быть выбрана
         * @return построитель
         */
        public Builder setSelectedRowIndex(Integer selectedRowIndex) {
            this.selectedRowIndex = selectedRowIndex;
            return this;
        }

        /**
         * Устанавливает описание действий на форме.
         *
         * @param actions описание действий на форме
         * @return построитель
         */
        public Builder setActions(List<TableSelectionFormAction> actions) {
            this.actions = actions;
            return this;
        }

        /**
         * Устанавливает идентификатор действия (кнопки на POS-кассе), которое должно быть выбрано при открытии формы.
         *
         * @param selectedActionId идентификатор действия (кнопки на POS-кассе), которое должно быть выбрано при открытии формы
         * @return построитель
         */
        public Builder setSelectedActionId(String selectedActionId) {
            this.selectedActionId = selectedActionId;
            return this;
        }

        /**
         * Возвращает набор параметров для открытия формы с таблицей, поддерживающей выбор строк.
         *
         * @return набор параметров для открытия формы с таблицей, поддерживающей выбор строк
         */
        public TableSelectionFormParameters build() {
            return new TableSelectionFormParameters(title, text, columns, rows, selectedRowIndex, actions, selectedActionId);
        }
    }
}
