package ru.crystals.pos.spi.ui;

import ru.crystals.pos.api.ui.listener.CancelListener;
import ru.crystals.pos.api.ui.listener.ConfirmListener;
import ru.crystals.pos.api.ui.listener.DialogListener;
import ru.crystals.pos.api.ui.listener.TableSelectionFormListener;
import ru.crystals.pos.api.ui.listener.TimeoutListener;
import ru.crystals.pos.spi.IncorrectStateException;
import ru.crystals.pos.spi.Injectable;
import ru.crystals.pos.spi.receipt.InfoPanelFormParameters;
import ru.crystals.pos.spi.ui.input.UIInputForms;
import ru.crystals.pos.spi.ui.payment.UIPaymentForms;
import ru.crystals.pos.spi.ui.webview.UIWebViewForms;

/**
 * Интерфейс для взаимодействия с пользовательским интерфейсом кассы.
 */
public interface UIForms extends Injectable {

    /**
     * Формы ввода данных кассы.
     *
     * @return UIInputForms
     */
    UIInputForms getInputForms();

    /**
     * Формы для плагинов оплат.
     *
     * @return UIPaymentForms
     */
    UIPaymentForms getPaymentForms();

    /**
     * Формы для WebView
     *
     * @return UIWebViewForms
     */
    UIWebViewForms getWebViewForms();

    /**
     * Отображает форму ожидания длительного процесса, который должен завершиться не более чем за указанное время.<br>
     * Данная форма используется в случае, когда необходимо показать окно ожидания выполнения длительного процесса, но притом сообщить пользователю,
     * сколько ещё времени (ориентировочно) ему предстоит подождать. Для этого пользователю отображается счетчик с обратным отсчетом и информационное
     * сообщение. По истечению отведенного времени окно может сообщать об этом, зажигая соответствующее событие через слушатель.
     *
     * @param message         информационное сообщение, которое отображается пользователю в процессе показа окна.
     * @param timeoutMs       начальное значение счетчика обратного отсчета. Задаётся в миллисекундах.
     * @param timeoutListener слушатель события обнуления счетчика. Событие носит информационный характер.
     */
    void showTimingOutForm(String message, int timeoutMs, TimeoutListener timeoutListener);

    /**
     * Показать форму ожидания длительного процесса.<br>
     * На экране кассы отображается анимированный спинер и текст сообщения. Используется при длительных операциях, когда необходимо дождаться результата и процесс прерывать нельзя.
     * <h3>Важно:</h3> Во время отображения данной формы весь пользовательский ввод (клавиатура, сканер и пр.) блокируется.
     * 
     * @param message
     *            сообщение на форме ожидания
     * @throws IncorrectStateException
     *             если невозможно отобразить форму в данный момент
     */
    void showSpinnerForm(String message) throws IncorrectStateException;

    /**
     * Показать форму ожидания длительного процесса, с возможностью отмены пользователем.<br>
     * На экране кассы отображается анимированный спинер и текст сообщения. Кассир может нажать кнопку {@code<Отмена>}.
     * 
     * @param message
     *            сообщение на форме ожидания
     * @param canceledFormListener
     *            получатель событий формы
     * @throws IncorrectStateException
     *             если невозможно отобразить форму в данный момент
     */
    void showSpinnerFormWithCancel(String message, CancelListener canceledFormListener) throws IncorrectStateException;

    /**
     * Показать экранную форму с сообщением об ошибке и ожиданием подтверждения после прочтения.
     * 
     * @param errorText
     *            текст ошибки
     * @param confirmListener
     *            получатель событий формы
     * @throws IncorrectStateException
     *             если невозможно отобразить форму в данный момент
     */
    void showErrorForm(String errorText, ConfirmListener confirmListener) throws IncorrectStateException;

    /**
     * Отображает экранную форму с текстовым сообщением.
     * @param message текст сообщения
     * @param confirmListener получатель событий формы
     * @throws IncorrectStateException если невозможно отобразить форму в данный момент
     */
    void showMessageForm(String message, ConfirmListener confirmListener) throws IncorrectStateException;

    /**
     * Показать форму диалога с двумя вариантами выбора (кнопками).<br>
     * Пользователь (кассир) также может отказаться от выбора и нажать клавишу <b>Отмена</b> 
     * (или попытаться закрыть форму),
     * в таком случае будет отправлено событие {@link DialogListener#eventCanceled()}.
     *
     * @param dialogFormParameters
     *            параметры формы
     * @param dialogListener
     *            получатель событий формы
     * @throws IncorrectStateException
     *             если невозможно отобразить форму в данный момент
     */
    void showDialogForm(DialogFormParameters dialogFormParameters, DialogListener dialogListener) throws IncorrectStateException;

    /**
     * Показать форму с таблицей
     *
     * @param parameters      параметры формы
     * @param confirmListener получатель событий формы
     * @throws IncorrectStateException если невозможно отобразить форму в данный момент
     */
    default void showTableForm(TableFormParameters parameters, ConfirmListener confirmListener) throws IncorrectStateException {
        throw new UnsupportedOperationException();
    }

    /**
     * Показать форму с таблицей, поддерживающей выбор строк.
     *
     * @param parameters параметры формы
     * @param listener   получатель событий формы
     * @throws IncorrectStateException если невозможно отобразить форму в данный момент
     * @since 1.1.0
     */
    default void showTableSelectionForm(TableSelectionFormParameters parameters, TableSelectionFormListener listener) throws IncorrectStateException {
        throw new UnsupportedOperationException();
    }

    /**
     * Показать форму с информационной панелью
     *
     * @param parameters      параметры формы
     * @param confirmListener получатель событий формы
     * @throws IncorrectStateException если невозможно отобразить форму в данный момент
     */
    default void showInfoPanelForm(InfoPanelFormParameters parameters, ConfirmListener confirmListener) throws IncorrectStateException {
        throw new UnsupportedOperationException();
    }
}
